local MAJOR = "LibQTipClick-1.1"
local MINOR =  1
assert(LibStub, MAJOR.." requires LibStub")

local lib, oldminor = LibStub:NewLibrary(MAJOR, MINOR)

if not lib then return end -- No upgrade needed

local QTip = LibStub:GetLibrary("LibQTip-1.0")
assert(QTip, MAJOR.." requires LibQTip-1.0")

local CBH = LibStub:GetLibrary("CallbackHandler-1.0")
assert(CBH, MAJOR.." requires CallbackHandler-1.0")

-------------------------------------------------------------------------------
-- Local variables
-------------------------------------------------------------------------------
lib.callbacks = lib.callbacks or CBH:New(lib, "SetCallback", "UnSetCallback", false)

lib.LabelProvider, lib.LabelPrototype, lib.BaseProvider = QTip:CreateCellProvider(QTip.LabelProvider)
local cell_provider, cell_prototype, cell_base = lib.LabelProvider, lib.LabelPrototype, lib.BaseProvider

-------------------------------------------------------------------------------
-- Public library API
-------------------------------------------------------------------------------
function lib.OnEnter(event, cell, arg)
	if not cell.highlight then
		cell.highlight = cell:CreateTexture(nil, "BACKGROUND")
		cell.highlight:Hide()
	end
	cell.highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	cell.highlight:SetBlendMode("ADD")
	cell.highlight:SetAllPoints(cell)
	cell.highlight:Show()
end

function lib.OnLeave(event, cell, arg)
	if cell.highlight then
		cell.highlight:ClearAllPoints()
		cell.highlight:Hide()
	end
end

function lib.OnMouseDown(event, cell, arg, button) DEFAULT_CHAT_FRAME:AddMessage(MAJOR..": No function defined.") end

local function Cell_OnEnter(cell) lib.callbacks:Fire("OnEnter", cell, cell.arg) end
local function Cell_OnLeave(cell) lib.callbacks:Fire("OnLeave", cell, cell.arg) end
local function Cell_OnMouseDown(cell, button) lib.callbacks:Fire("OnMouseDown", cell, cell.arg, button) end

function cell_prototype:InitializeCell() cell_base.InitializeCell(self) end

function cell_prototype:SetupCell(tooltip, value, justification, font, arg, ...)
	local width, height = cell_base.SetupCell(self, tooltip, value, justification, font, arg, ...)

	self:EnableMouse(true)
	self.arg = arg
	self:SetScript("OnEnter", Cell_OnEnter)
	self:SetScript("OnLeave", Cell_OnLeave)
	self:SetScript("OnMouseDown", Cell_OnMouseDown)

	return width, height
end

function cell_prototype:ReleaseCell()
	self:EnableMouse(false)
	self:SetScript("OnEnter", nil)
	self:SetScript("OnLeave", nil)
	self:SetScript("OnMouseDown", nil)
	self.arg = nil
end

-------------------------------------------------------------------------------
-- LibQTip wrapper API
-------------------------------------------------------------------------------
local function AddNormalLine(tooltip, ...)
	local oldProvider = tooltip:GetDefaultProvider()
	tooltip:SetDefaultProvider(QTip.LabelProvider)
	local lineNum, colNum = tooltip:AddLine(...)
	tooltip:SetDefaultProvider(oldProvider)
	return lineNum, colNum
end

local function AddNormalHeader(tooltip, ...)
	local oldProvider = tooltip:GetDefaultProvider()
	tooltip:SetDefaultProvider(QTip.LabelProvider)
	local lineNum, colNum = tooltip:AddHeader(...)
	tooltip:SetDefaultProvider(oldProvider)
	return lineNum, colNum
end

local function SetNormalCell(tooltip, ...)
	local oldProvider = tooltip:GetDefaultProvider()
	tooltip:SetDefaultProvider(QTip.LabelProvider)
	local lineNum, colNum = tooltip:SetCell(...)
	tooltip:SetDefaultProvider(oldProvider)
	return lineNum, colNum
end

function lib:Acquire(key, ...)
	local tooltip = QTip:Acquire(key, ...)
	tooltip:EnableMouse(true)

	self.SetCallback(tooltip, "OnEnter", self.OnEnter)
	self.SetCallback(tooltip, "OnLeave", self.OnLeave)
	self.SetCallback(tooltip, "OnMouseDown", self.OnMouseDown)
	tooltip.AddNormalLine = AddNormalLine
	tooltip.AddNormalHeader = AddNormalHeader
	tooltip.SetNormalCell = SetNormalCell
	tooltip:SetDefaultProvider(cell_provider)

	return tooltip
end

function lib:IsAcquired(key) return QTip:IsAcquired(key) end

function lib:Release(tooltip)
	tooltip:EnableMouse(false)
	QTip:Release(tooltip)
end

function lib:IterateTooltips() return QTip:IterateTooltips() end
function lib:CreateCellProvider(baseProvider) return QTip:CreateCellProvider(baseProvider) end
